package com.ruoyi.brewery.service;

import java.util.List;
import com.ruoyi.brewery.domain.WyBeerReview;
import com.ruoyi.common.core.domain.entity.WyBeerReviewScore;

/**
 * 酒评Service接口
 *
 * @author ruoyi
 * @date 2024-12-21
 */
public interface IWyBeerReviewService
{
    /**
     * 查询酒评
     *
     * @param id 酒评主键
     * @return 酒评
     */
    public WyBeerReview selectWyBeerReviewById(Long id);

    int setWyBeerReviewTop(Long id);

    int cancelWyBeerReviewTop(Long id);

    WyBeerReview selectWyBeerReviewByBeerIdTop1(Long beerId);

    WyBeerReview selectWyBeerReviewByBeerIdAndCustomerId(Long beerId, Long customerId);

    /**
     * 查询酒评列表
     *
     * @param wyBeerReview 酒评
     * @return 酒评集合
     */
    public List<WyBeerReview> selectWyBeerReviewList(WyBeerReview wyBeerReview);

    List<WyBeerReview> selectWyBeerReviewListWithLike(WyBeerReview wyBeerReview, Long userId);

    WyBeerReviewScore selectWyBeerReviewScoreList(Long beerId);

    List<WyBeerReviewScore> selectWyBeerReviewScoreAll();

    /**
     * 新增酒评
     *
     * @param wyBeerReview 酒评
     * @return 结果
     */
    public int insertWyBeerReview(WyBeerReview wyBeerReview);

    /**
     * 修改酒评
     *
     * @param wyBeerReview 酒评
     * @return 结果
     */
    public int updateWyBeerReview(WyBeerReview wyBeerReview);

    /**
     * 批量删除酒评
     *
     * @param ids 需要删除的酒评主键集合
     * @return 结果
     */
    public int deleteWyBeerReviewByIds(Long[] ids);

    /**
     * 删除酒评信息
     *
     * @param id 酒评主键
     * @return 结果
     */
    public int deleteWyBeerReviewById(Long id);

    Integer selectBeerReviewCountByDays(Long breweryId, int lastDays);

    List<WyBeerReview> selectWyBeerReviewListWithBrewery(WyBeerReview wyBeerReview);
}
